/**
 * Copyright 2016 Rice University
 *
 *  Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 *  the License. You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software distributed under the License is distributed
 *  on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for
 *  the specific language governing permissions and limitations under the License.
 */
part of controllers;

/**
 * A component of IDE functionality that must be started prior to providing functionality.  Most commonly
 * coordinates the operation of a View and ViewModel.
 *
 * To start the component invoke the [start] method.
 */
abstract class Controller
{
  /**
   * Indicates whether this controller has previously been started.
   */
  bool _hasStarted = false;

  /**
   * Indicates whether this controller has previously been started.
   */
  bool get hasStarted => _hasStarted;

  /**
   * Starts the component.  May only be called once per instance.  Subsequent invocations result
   * in the raising of a [StateError] exception.
   */
  void start()
  {
    if(_hasStarted)
      throw new StateError("Controller has already been started.");

    _hasStarted = true;

  }

  /**
   * Disables the functionality provided by this component until the [resume] method is invoked.
   */
  void suspend();

  /**
   * Restores the functionality of this component if previously suspended by the [suspend] method.
   */
  void resume();

  /**
   * Throws [StateError] if the controller had not been started.
   */
  void assertHasStarted()
  {
    if(!_hasStarted)
      throw new StateError("Controller has not been started");
  }
}